option spool;
 /*****************************************************************/
 /* NAME:  GEOMACRO                                               */
 /* DESCRIPTION: program to create the geographic analysis gif    */
 /*   and HTML files using parameters from the property sheet     */
 /* PRODUCT:  HR Vision                                           */
 /* USAGE: %geomacro                                              */
 /*****************************************************************/

%macro geomacro(hrvurl,
		var,
             	stat,
             	statlbls,
             	statdata,
		mapdata,
             	outGif,
             	outHtml,
             	outFile,
             	prvloc,
           	DSN,
		linkvar,
		dispvar,
		nextlevel,
		where,
		thiskey,
		intloc,
		slctvar,
		prevlevel,
		desc,
		count,
		legendLevels,
		loc,
		geoWhere1,
		geoWhere2,
		geoWhere3,
		geoWhere4,
		geoWhere5,
		datasrc);
%local i j k label dsid onestat onevar statlbl;
/*****************************************************************/
/* determine if there is an annotate data set to include on the  */
/* gmap statement. If so, derive the data set name and insert it */
/* into a macro variable                                         */
/*****************************************************************/
%local annotate anno;
%let anno=anno;
%if %sysfunc(exist(&mapdata&anno))=1 %then %do;
      %let annotate=annotate=&mapdata&anno;
%end;

/*****************************************************************/
/* parse the var and stat parameters to get a count then use the */
/* count to create macro variables to hold the stat-names and the*/
/* stat-name labels                                              */
/*****************************************************************/
%let statline=;
%let i=0;
%let k=0;
%let labelcount=0;
%let dvarcount=0;
%let dsid=%sysfunc(open(&statdata));
%do %until(&onestat=);
  %let i=%eval(&i+1);
  %let onestat=%scan(&stat,&i,%str(~ ,.*&^%$#@));
  %let statlbl=%nrbquote(%scan(&statlbls,&i,%str('~')));
%if &onestat ne %then %do;
    %let j=0;
    %do %until(&onevar=);
      %let j=%eval(&j+1);
        %let onevar=%scan(&var,&j);
        %if &onevar ne %then %do;
        %let labelcount=%eval(&labelcount+1);
        %let varnum=%sysfunc(varnum(&dsid,&onevar));
          %if &dsid gt 0 and &varnum gt 0 %then 
              %let label=%nrbquote(%sysfunc(varlabel(&dsid,&varnum)));
          %else %let label=%nrbquote(&onevar);
          %if &onevar ne %str() and &onestat ne %str() %then %do;
            %global name&labelcount label&labelcount;
            %let name&labelcount=&onestat&onevar;
              %if &j=1 %then
              %let statline=&statline &onestat=&&name&labelcount;
              %else %let statline=&statline &&name&labelcount;
          %let dash=-;
          %let label&labelcount=%nrbquote(&statlbl &dash &label);
        %end;
      %end;
    %end;
  %end;
%end;
/* loop through the dispvar (display var) field, */
/* create the separate display fields         */
%do %until(&dvar=);
  %let k=%eval(&k+1);
  %let dvar=%scan(&dispvar,&k,%str(~ ,.*&^%$#@));
  %if &dvar ne %then %do;
 	%let dvarcount=%eval(&dvarcount+1);
    %let varnum=%sysfunc(varnum(&dsid,&dvar));
      %if &dsid gt 0 and &varnum gt 0 %then %do;
		%let label=%nrbquote(%sysfunc(varlabel(&dsid,&varnum)));
		%let fmt=%sysfunc(varfmt(&dsid,&varnum));
	  %end;
      %else %do;
		%let label=%nrbquote(&dvar);
		%let fmt=;
	  %end;
    %if &dvar ne %str() %then %do;
      %global dname&dvarcount dlabel&dvarcount dfmt&dvarcount;
      %let dname&dvarcount=&dvar;
      %let dlabel&dvarcount=%nrbquote(&label);
      %let dfmt&dvarcount=&fmt;
    %end;
  %end;
%end;
%if &dsid gt 0 %then %let dsid=%sysfunc(close(&dsid));

%let basepath = &outGif;
%let newdir =;
proc display c=proxy.proxy.dcreate.scl batch; run;
%let basepath =;


 /* Create an HTML file to display the map gif. */
options notes;
filename outGif "&outGif";
filename outFile &outFile;
filename outHtml &outHtml;

 
/* run a proc means on pubdata.empmast */
proc means data=&statdata noprint nway;
class &linkvar;
id &dispvar;
var &var;
output out=work.meansout &statline ;
run;
/* add the url/drill info to the stats */
data mapstats(rename=(_freq_=Count));
  attrib url length=$1000;
  set meansout;
  label _freq_ = "&count";
  format _freq_ comma12.;
  
  attrib string length=$40;
  attrib qstring length=$40;
  attrib remaining length=$40;
  stop=0;
  start=1;
  stop=index(%nrquote(&linkvar),"'");
  if stop > 0 then do;
	  do while (stop > 0);
	    if stop=1 then do;
	      string="\'\'";
	      qstring="''";
	    end;
	    else do;
	      string = trim(string)||(trim(substr(%nrquote(&linkvar),start,stop-start)))||"\'\'";
	      qstring = trim(qstring)||(trim(substr(%nrquote(&linkvar),start,stop-start)))||"''";
	    end;
	    if stop = length(&linkvar) then do;
	      remaining='';
	      stop=0;  
	    end;
	    else do;
	      start = stop +1;
	      remaining = substr(%nrquote(&linkvar),start);
	      check=index(%nrquote(remaining),"'");
	      if check = 0 then stop = 0;
	      else stop = check + start-1;
            end;
	  end;
       string = LEFT(trim(string)||trim(remaining));
       qstring = LEFT(trim(qstring)||trim(remaining));
  end;
  else do;
    string = &linkvar;
    qstring = &linkvar;
  end;
  
  geowhere1=scan(substr(%nrquote("&geowhere1"),2),1,')');
  geowhere2=scan(substr(%nrquote("&geowhere2"),2),1,')');
  geowhere3=scan(substr(%nrquote("&geowhere3"),2),1,')');
  geowhere4=scan(substr(%nrquote("&geowhere4"),2),1,')');
  geowhere5=scan(substr(%nrquote("&geowhere5"),2),1,')');
  
  if "&nextLevel" = "null" or "&nextLevel" = "" then
      url = 'href="javaScript:parent.parent.ICONS.processItemClick('||
					"'',"||
					"'',"||
					"'"||"&linkvar"||"',"||
					"'"||trim(string)||"',"||
					"'"||"&datasrc"||"',"||
					"false,'',"||&loc||","||
					"'"||trim(geoWhere1)||"',"||
					"'"||trim(geoWhere2)||"',"||
					"'"||trim(geoWhere3)||"',"||
					"'"||trim(geoWhere4)||"',"||
					"'"||trim(geoWhere5)||"'"||
					");areaMouseOut('"||trim(string)||"')"||
					'" onMouseOver="areaMouseOver('||"'"||trim(string)||"'"||');'||
					'" onMouseOut="areaMouseOut('||"'"||trim(string)||"'"||');"';
  else
  url = 'href="javaScript:parent.parent.ICONS.processItemClick('||
			"'"||"&thisKey"||"',"||
			"'"||"&nextLevel"||"',"||
			"'"||"&linkvar"||"',"||
			"'"||trim(string)||"',"||
			"'"||"&datasrc"||"',"||
			"true,'"||"&intloc"||"',"||"&loc"||","||
			"'"||trim(geoWhere1)||"',"||
			"'"||trim(geoWhere2)||"',"||
			"'"||trim(geoWhere3)||"',"||
			"'"||trim(geoWhere4)||"',"||
			"'"||trim(geoWhere5)||"'"||					
			");areaMouseOut('"||trim(string)||"')"||
			'" onMouseOver="areaMouseOver('||"'"||trim(string)||"'"||');'||
			'" onMouseOut="areaMouseOut('||"'"||trim(string)||"'"||');"';			
run;

/* stops sending output to GRAPH and OUTPUT windows */
ods listing close;

/* Reset the goptions  */
goptions reset=global;

/* set graphics options for ODS HTML output */
goptions device=gif gsfname=outGif transparency noborder 
         colors=(red yellow blue green cyan magenta pink orange gray black )
         ctext=black;

%let styletxt=%str(<link href=%"&hrvurl/Html/sasads.css%" rel=%"STYLESHEET%" type=%"text/css%"><link href=%"&hrvurl/Html/sasStyle.css%" rel=%"STYLESHEET%" type=%"text/css%"><link href=%"&hrvurl/Html/HRVstyles.css%" rel=%"STYLESHEET%" type=%"text/css%">);


/* use define style to apply the stylesheets.                */
proc template;

define Style styles.HRVstyle; parent = styles.default;

      /* ----- */
   Replace Document from Container "Abstract. Controls the various document bodies." /
      csstext = "&styletxt";
 
   ;

end;

run;

/* Create the text to be used in the HEADTEXT statement. This *
 * is text which will be included between the head tags of    *
 * the HTML.                                                  */
 
 %let hdtext=%str(<META HTTP-EQUIV=%"Pragma%" CONTENT=%"no-cache%">
 <script language=%"JavaScript%" src=%"&hrvurl/Html/areaMouseCtrl.js%"></script>);
 
 %let urltext=ViewFileServlet?type=analysis%nrstr(&ext)=gif%nrstr(&key)=&thiskey%nrstr(&outloc)=&prvloc%nrstr(&relpath)=&thiskey%nrstr(&remote)=Y%nrstr(&location)=&intloc%nrstr(&file)=;

ods html style=HRVstyle gpath="&outGif"(url="&urltext") body=outFile (no_bottom_matter) headtext="&hdtext";

/* delete gif entries from prior runs */
%delgif;

/* run proc gmap to attach stats to geographic areas */

/* ### still need to decide about number of color levels, legend, etc ### */
/* ### still need to decide about number of color levels, legend, etc ### */

proc gmap map=&mapdata &WHERE data=work.mapstats all;
	legend1 across=1;
   id &linkvar;
   choro Count / html=url name="&DSN" levels=&legendLevels coutline=black
                 ctext=black cempty=black legend=legend1 &annotate;
   pattern1 value=msolid;
run;
quit;
ods html close;
filename outFile &outFile mod;
/* write out the HTML to populate the stats box displayed onMouseOver */
data _null_;
  set mapstats end=last;
  format &linkvar;
file outfile;
if Count ne . then do;
put '      <table id="'qstring +(-1)'" cellpadding="0" cellspacing="0" border="0"';
put '            class="GeoDetail" style="position:absolute; display:none;">';
%do i=1 %to &&dvarCount;
put '            <tr><td class="GeoLabel">&nbsp;' %nrquote("&&dlabel&i") ':&nbsp;</td>';
put '                  <td class="GeoData">&nbsp;'&&dname&i &&dfmt&i '&nbsp;</td></tr>';
%end;
%do i=1 %to &labelCount;
    %if %substr(&&name&i,1,1) = N %then %do;
    	%global numfmt;
    	%if &numfmt=%str() %then %let cntfmt=comma12.;
    	%else %let cntfmt=%scan(&numfmt,1,'.').; 
		format &&name&i &cntfmt;
	%end;
      put '            <tr><td class="GeoLabel">&nbsp;' %nrquote("&&label&i") ':&nbsp;</td>';
      put '                  <td class="GeoData">&nbsp;' &&name&i '&nbsp;</td></tr>';
%end;

put '      </table>';
end;

if last then do;
	if "&prevlevel" ne "null" then
		put '<form name="geostate"><input type="hidden" name="hasprev" value="Y"></form>'; 
	else put '<form name="geostate"><input type="hidden" name="hasprev" value="N"></form>';
	
	put '<script language="JavaScript"> parent.BUTTONS.location=parent.BUTTONS.location;</script>';

end;

run;

ods html style=HRVstyle gpath="&outGif"(url="&urltext") body=outFile (no_top_matter) anchor='end';

ods html close;

ods html  gpath="&outGif"(url="&urltext") body=final(no_top_matter no_bottom_matter);

filename final &outHtml lrecl=10000;
    
%let maxlen = 10000;
    
data _null_;
	infile outFile sharebuffers length=linelen lrecl=&maxlen;
	file final;
	input @;
	if linelen gt 0 then
    input @ 1 record $varying&maxlen.. linelen;
	put _infile_ ; 
run;
  
    
ods html close;

ods listing;

filename outGif clear;
filename outFile clear;
filename outHtml clear;
filename final clear;

%mend geomacro;

/* macro to delete the grseg/gif entry every time */
%macro delgif;
%if %sysfunc(cexist(work.gseg.&DSN..grseg))=1 %then %do;
      proc greplay igout=work.gseg nofs;
            delete &DSN;
      run;
      quit;
%end;
%mend delgif;
