/* Security macro to read the users base data and create a macro variable for each one. */
%macro secvars(id=,baseds=pubdata.empmast,idvar=ID,secbase=_SEC_);
  %local dsid rc varname nvars nodatarc;
  %global debug;
  %let dsid=%sysfunc(open(&baseds(where=(&idvar="&id") &datapass)));
  
  /* validate that the base dataset can be opened, has variables, and has a record for the ID value */
  %if &dsid le 0 %then %do;
	  %put WARNING: Cannot open &baseds with &idvar="&id". No security subset macro variables can be created.;
          %goto done;
  %end;
  %let nvars=%sysfunc(attrn(&dsid,NVARS));
  %if &nvars le 0 %then %do;
	  %put WARNING: No variables in &baseds.. No security subset macro variables can be created.;
    	  %goto done;
  %end;
  %let nodatarc=%sysfunc(fetch(&dsid));
  %if &nodatarc ne 0 %then %do;
	  %put WARNING: Cannot find data for &idvar="&id" in &baseds.. No security subset macro variables can be created.;
	  %goto done;
  %end;

  %if %upcase(&debug)=Y %then %do;
    %put NOTE: Creating security macro variables for &idvar="&id".;
  %end;

  /* for each value for the current record, transfer to a macro variable with the same name */
  %do i=1 %to &nvars;
    %let varname=%unquote(&secbase%upcase(%sysfunc(varname(&dsid,&i))));
    %global &varname;

    %if &nodatarc ne 0 %then 
      %let &varname=;
    %else %if %sysfunc(vartype(&dsid,&i))=C %then  
      %let &varname=%qsysfunc(getvarc(&dsid,&i));
    %else 
      %let &varname=%qsysfunc(getvarn(&dsid,&i)); 
    %if %upcase(&debug)=Y %then %put NOTE: Security macro variable created: &varname = &&&varname;
  %end;

  %put NOTE: Security subset macro variables have been created.;
%done:
  %if &dsid gt 0 %then %let dsid=%sysfunc(close(&dsid));
%mend;